@@ -4,6 +4,7 @@ Utility functions for strings, paths and URL.
 
 import re
 import json
+from .constants import REGEX_FILE_ID
 
 __all__ = [
     'assert_file_grp_cardinality',
@@ -79,6 +80,10 @@ def make_file_id(ocrd_file, output_file_grp):
         while ocrd_file.mets.find_files(ID=ret):
             n += 1
             ret = concat_padded(output_file_grp, n)
+    if not REGEX_FILE_ID.fullmatch(ret):
+        ret = ret.replace(':', '_')
+        ret = re.sub(r'^([^a-zA-Z_])', r'id_\1', ret)
+        ret = re.sub(r'[^\w.-]', r'', ret)
     return ret
 
 def nth_url_segment(url, n=-1):
@@ -172,4 +177,3 @@ def safe_filename(url):
     ret = re.sub('[^A-Za-z0-9]+', '.', url)
     #  print('safe filename: %s -> %s' % (url, ret))
     return ret
-
